package fr.inway;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.xsocket.connection.IConnectHandler;
import org.xsocket.connection.IDataHandler;
import org.xsocket.connection.IDisconnectHandler;
import org.xsocket.connection.INonBlockingConnection;

public class PolicyProtocolHandler implements IConnectHandler, IDisconnectHandler, IDataHandler {
    protected String policyData;
    protected boolean showTrace = false;;
    protected List<INonBlockingConnection> connections;
    
	public PolicyProtocolHandler(String policyData, boolean showTrace)
	{
	    this.policyData = policyData;
	    this.showTrace = showTrace;
	    connections = new ArrayList<INonBlockingConnection>();
	}

    public boolean onConnect(INonBlockingConnection connection)
    {
    	/*
        boolean b = false;
        try
        {
            connection.setIdleTimeoutMillis(10*1000);
            connection.setAutoflush(false);
            synchronized(connections)
            {
                connections.add(connection);
            }
            updateGui(connections.size());
            //
            connection.write(policyData);
            connection.write((int)0);
            connection.flush();
            b = true;
        }
        catch(IOException ex)
        {
            Logger.getLogger(PolicyProtocolHandler.class.getName()).log(Level.SEVERE, null, ex);
        }
        return b;
        */
        if (showTrace) System.out.println("Connexion by "+connection.getRemoteAddress());
        connection.setIdleTimeoutMillis(10*1000);
        connection.setAutoflush(false);
        synchronized(connections)
        {
            connections.add(connection);
        }
        updateGui(connections.size());
        return true;

    }

    public boolean onData(INonBlockingConnection connection)
    {
    	//System.out.println("onData");
        boolean b = false;
        if (!connection.isOpen()) return false;
        try
        {
        	String data = connection.readStringByDelimiter("\0");
        	if(data != null && data.length()> 0)
            {  
        		data = data.trim();
                if (showTrace) System.out.println("Incoming data: " + data);
                if(data.equalsIgnoreCase("<policy-file-request/>"))
                {
                    connection.write(policyData);
                    connection.flush();
                    connection.close();
                }
            }      	
        	else
        	{
                if (showTrace) System.out.println("read data NULL");       		
        	}
            b = true;
        }
        catch(IOException ex)
        {
            Logger.getLogger(PolicyProtocolHandler.class.getName()).log(Level.SEVERE, null, ex);
        }
        return b;
    }
    
    public boolean onDisconnect(INonBlockingConnection connection)
    {
    	//System.out.println("onDisconnect");
        synchronized(connections)
        {
            connections.remove(connection);
        }
        updateGui(connections.size());
        return true;
    }

    private void updateGui(int count)
    {
        if(Main.server.getGUI() != null)
        {
            Main.server.getGUI().updateCount(count);
        }
    }
}
